@@ -1,7 +1,7 @@
 """This module contains helpers for the collection package."""
 from __future__ import annotations
-from abc import ABC, abstractmethod
 from collections import defaultdict
+from datetime import datetime
 import logging
 from operator import itemgetter
 from pathlib import Path
@@ -11,8 +11,10 @@ from typing import Dict, List, Optional, Set, Tuple, Union
 
 from djtools.collection.collections import Collection, RekordboxCollection
 from djtools.collection.config import PlaylistConfig, PlaylistConfigContent
+from djtools.collection.playlist_filters import PlaylistFilter
 from djtools.collection.playlists import Playlist, RekordboxPlaylist
 from djtools.collection.tracks import Track
+from djtools.utils.helpers import make_path
 
 
 logger = logging.getLogger(__name__)
@@ -24,6 +26,7 @@ logger = logging.getLogger(__name__)
 # #############################################################################
 
 
+@make_path
 def copy_file(track: Track, destination: Path):
     """Copies a track to a destination and updates its location.
 
@@ -40,28 +43,24 @@ def copy_file(track: Track, destination: Path):
 
 # #############################################################################
 # This section includes helpers for the playlist_builder module.
-#   - PLATFROM_REGISTRY: used to determine which abstraction implementations to
+#   - PLATFORM_REGISTRY: used to determine which abstraction implementations to
 #       use e.g. "rekordbox"
 #   - build_tag_playlists: builds collection playlists using "tags" component
 #       of the PlaylistConfig
-#   - PlaylistFilter: abstraction for filtering Tracks from Playlists
-#   - HipHopFilter: used to distinguish between actual hip hop tracks and bass
-#       tracks that have hip hop influences
-#   - MinimalDeepTech: used to distinguish between minimal deep tech tracks
-#       with predominately house and techno influences
 #   - filter_tag_playlists: applies PlaylistFilter implementations to built tag
 #       playlists
 #   - aggregate_playlists: aggregates tracks from playlists in folders to
 #       create an "All <folder name>" playlist in each folder
-#   - add_selectors_to_tags: parses combiner playlist names and finds BPM,
-#       rating, and playlist selectors to update the tag -> track lookup
-#   - parse_bpms_and_ratings: used to parse BPM and rating selectors
+#   - add_selectors_to_tags: parses combiner playlist names and finds numerical
+#       and string selectors to update the tag -> track lookup
+#   - parse_numerical_selectors: used to parse numerical selectors like
+#       ratings, BPMs, and years
 #   - build_combiner_playlists: builds collection playlists using "combiner"
 #       component of the PlaylistConfig
 #   - parse_expression: evaluates the boolean algebra logic in combiner
 #       playlists names to populate them with the appropriate tracks
 #   - BooleanNode: used to build and evaluate the boolean algebra parse tree
-#   - print_playlist_tag_statistics: prints ASCII histograms showing tag
+#   - print_playlists_tag_statistics: prints ASCII histograms showing tag
 #       frequencies in combiner playlists split by genre and other tag types
 #   - scale_data: scales tag frequencies to normalize histogram height
 #   - print_data: formats the string representing ASCII histograms
@@ -84,7 +83,7 @@ def build_tag_playlists(
     tags_tracks: Dict[str, Dict[str, Track]],
     playlist_class: Playlist,
     tag_set: Optional[Set] = None,
-) -> Union[None, Playlist]:
+) -> Optional[Playlist]:
     """Recursively traverses a playlist config to generate playlists from tags.
 
     Args:
@@ -101,59 +100,12 @@ def build_tag_playlists(
     Returns:
         A Playlist or None.
     """
+    if not isinstance(content, (PlaylistConfigContent, str)):
+        raise ValueError(f"Invalid input type {type(content)}: {content}")
+
     # Initialize the set of tags in case the caller didn't provide one.
     tag_set = tag_set if tag_set is not None else set()
 
-    # This is not a folder so a playlist with tracks must be created.
-    if isinstance(content, str):
-        # Apply special logic for creating a "pure" playlist. "Pure" playlists
-        # are those that contain tracks with a set of genre tags that all
-        # contain the sub-string indicated by the suffix of the playlist name.
-        # For example, "Pure Techno" will contain tracks that have genres
-        # {"Hard Techno", "Melodic Techno"} but will not contain tracks that
-        # contain {"Hard Techno", "Tech House"} because "Tech House" does not
-        # contain "Techno" as a sub-string.
-        if content.startswith("Pure "):
-            # Isolate the tag to create a pure playlist for.
-            tag = content.split("Pure ")[-1]
-            tracks_with_tag = tags_tracks.get(tag)
-            if not tracks_with_tag:
-                logger.warning(
-                    f'Can\'t make a "Pure {tag}" playlist because there are '
-                    "no tracks with that tag."
-                )
-                return None
-
-            # Filter out tracks that aren't pure.
-            pure_tag_tracks = {
-                track_id: track for track_id, track in tracks_with_tag.items()
-                if all(
-                    tag.lower() in _.lower() for _ in track.get_genre_tags()
-                )
-            }
-            if not pure_tag_tracks:
-                logger.warning(
-                    f'Can\'t make a "Pure {tag}" playlist because there are '
-                    f"no tracks that are pure {tag}."
-                )
-                return None
-
-            return playlist_class.new_playlist(
-                name=content, tracks=pure_tag_tracks
-            )
-
-        # Get tracks with this tag and index it so that it's not added to the
-        # "Other" playlists.
-        tracks_with_tag = tags_tracks.get(content)
-        tag_set.add(content)
-        if not tracks_with_tag:
-            logger.warning(f'There are no tracks with the tag "{content}"')
-            return None
-
-        return playlist_class.new_playlist(
-            name=content, tracks=tracks_with_tag
-        )
-
     # This is a folder so create playlists for those playlists within it.
     if isinstance(content, PlaylistConfigContent):
         # Update the set of tags seen so these are ignored in when creating the
@@ -175,151 +127,59 @@ def build_tag_playlists(
             return None
 
         return playlist_class.new_playlist(
-            name=content.name, playlists=playlists,
+            name=content.name, playlists=playlists
         )
 
-    raise ValueError(f"Invalid input type {type(content)}: {content}")
+    # This is not a folder so a playlist with tracks must be created.
 
+    # Apply special logic for creating a "pure" playlist. "Pure" playlists are
+    # those that contain tracks with a set of genre tags that all contain the
+    # sub-string indicated by the suffix of the playlist name. For example,
+    # "Pure Techno" will contain tracks that have genres {"Hard Techno",
+    # "Melodic Techno"} but will not contain tracks that contain
+    # {"Hard Techno", "Tech House"} because "Tech House" does not contain
+    # "Techno" as a sub-string.
+    if content.startswith("Pure "):
+        # Isolate the tag to create a pure playlist for.
+        tag = content.split("Pure ")[-1]
+        tracks_with_tag = tags_tracks.get(tag)
+        if not tracks_with_tag:
+            logger.warning(
+                f'Can\'t make a "Pure {tag}" playlist because there are no '
+                "tracks with that tag."
+            )
+            return None
 
-class PlaylistFilter(ABC):
-    "This class defines an interface for filtering tracks from playlists."
+        # Filter out tracks that aren't pure.
+        pure_tag_tracks = {
+            track_id: track for track_id, track in tracks_with_tag.items()
+            if all(tag.lower() in _.lower() for _ in track.get_genre_tags())
+        }
+        if not pure_tag_tracks:
+            logger.warning(
+                f'Can\'t make a "Pure {tag}" playlist because there are no '
+                f"tracks that are pure {tag}."
+            )
+            return None
 
-    @abstractmethod
-    def filter_track(self, track: Track) -> bool:
-        """Returns True if this track should remain in the playlist.
+        return playlist_class.new_playlist(name=content, tracks=pure_tag_tracks)
 
-        Args:
-            track: Track object to apply filter to.
+    # Get tracks with this tag and index it so that it's not added to the
+    # "Other" playlists.
+    tracks_with_tag = tags_tracks.get(content)
+    tag_set.add(content)
+    if not tracks_with_tag:
+        logger.warning(f'There are no tracks with the tag "{content}"')
+        return None
 
-        Returns:
-            Whether or not this track should be included in the playlist.
-        """
-
-    @abstractmethod
-    def is_filter_playlist(self, playlist: Playlist) -> bool:
-        """Returns True if this playlist should be filtered.
-
-        Args:
-            playlist: Playlist object to potentially filter.
-
-        Returns:
-            Whether or not to filter this playlist.
-        """
-
-
-class HipHopFilter(PlaylistFilter):
-    'This class filters playlists called "Hip Hop".'
-
-    def filter_track(self, track: Track) -> bool:
-        """Returns True if this track should remain in the playlist.
-
-        If the playlist is not underneath a folder called "Bass", then this
-        track is filtered out unless it has exclusively "Hip Hop" and "R&B"
-        genre tags. If the playlist is underneath a folder called "Bass", then
-        this track is filtered out if it does have exclusively "Hip Hop" and
-        "R&B" genre tags.
-
-        Args:
-            track: Track object to apply filter to.
-
-        Returns:
-            Whether or not this track should be included in the playlist.
-        """
-        pure_hip_hop_with_other_tags = not self._bass_hip_hop and any(
-            "r&b" not in x.lower() and "hip hop" not in x.lower()
-            for x in track.get_genre_tags()
-        )
-        bass_hip_hop_without_other_tags = self._bass_hip_hop and all(
-            "r&b" in x.lower() or "hip hop" in x.lower()
-            for x in track.get_genre_tags()
-        )
-        if pure_hip_hop_with_other_tags or bass_hip_hop_without_other_tags:
-            return False
-
-        return True
-
-
-    def is_filter_playlist(self, playlist: Playlist) -> bool:
-        """Returns True if this playlist's name is "Hip Hop".
-
-        Args:
-            playlist: Playlist object to potentially filter.
-
-        Returns:
-            Whether or not to filter this playlist.
-        """
-        if not playlist.get_name() == "Hip Hop":
-            return False
-
-        self._bass_hip_hop = False  #pylint: disable=attribute-defined-outside-init
-        parent = playlist.get_parent()
-        while parent:
-            if parent.get_name() == "Bass":
-                self._bass_hip_hop = True  #pylint: disable=attribute-defined-outside-init
-            parent = parent.get_parent()
-
-        return True
-
-
-class MinimalDeepTechFilter(PlaylistFilter):
-    'This class filters playlists called "Minimal Deep Tech".'
-
-    def filter_track(self, track: Track) -> bool:
-        """Returns True if this track should remain in the playlist.
-
-        If the playlist is not underneath a folder called "Techno", then this
-        track is filtered out if the genre tag preceding "Minimal Deep Tech" is
-        "Techno". If the playlist is underneath a folder called "Techno", then
-        this track is filtered out if the genre tag preceding
-        "Minimal Deep Tech" is not "Techno".
-
-        Args:
-            track: Track object to apply filter to.
-
-        Returns:
-            Whether or not this track should be included in the playlist.
-        """
-        genre_tags = track.get_genre_tags()
-        index = genre_tags.index("Minimal Deep Tech")
-        prefix_tag = genre_tags[index - 1]
-        techno_with_no_techno_prefix = (
-            self._techno and prefix_tag.lower() != "techno"
-        )
-        not_techno_with_techno_prefix = (
-            not self._techno and prefix_tag.lower() == "techno"
-        )
-        if techno_with_no_techno_prefix or not_techno_with_techno_prefix:
-            return False
-
-        return True
-
-    def is_filter_playlist(self, playlist: Playlist) -> bool:
-        """Returns True if this playlist's name is "Minimal Deep Tech".
-
-        Args:
-            playlist: Playlist object to potentially filter.
-
-        Returns:
-            Whether or not to filter this playlist.
-        """
-        if not playlist.get_name() == "Minimal Deep Tech":
-            return False
-
-        self._techno = False  #pylint: disable=attribute-defined-outside-init
-        parent = playlist.get_parent()
-        while parent:
-            if parent.get_name() == "Techno":
-                self._techno = True  #pylint: disable=attribute-defined-outside-init
-            parent = parent.get_parent()
-
-        return True
+    return playlist_class.new_playlist(name=content, tracks=tracks_with_tag)
 
 
 def filter_tag_playlists(
     playlist: Playlist, playlist_filters: List[PlaylistFilter]
 ) -> None:
     """Applies a list of PlaylistFilter implementations to the playlist.
-    
+
     If the PlaylistFilter implementations' is_filter_playlist method evaluates
     to True, then the filter_track method is applied to each track in the
     playlist. The playlist's tracks are set to remove the tracks that have been
@@ -388,143 +248,285 @@ def aggregate_playlists(
 
 
 def add_selectors_to_tags(
-    playlist_config: PlaylistConfig,
+    content: Union[PlaylistConfigContent, str],
     tags_tracks: Dict[str, Dict[str, Track]],
     collection: Collection,
     auto_playlists: List[Playlist],
 ):
-    """Parse the Combiner playlists for selectors and update track lookup.
+    """Recursively update the track lookup with selectors.
 
     Args:
-        playlist_config: PlaylistConfig object.
+        content: A component of a playlist config to create a playlist for.
         tags_tracks: Dict of tags to tracks.
         collection: Collection object.
         auto_playlists: Tag playlists built in this same run.
     """
+    # This is a folder so parse selectors from playlists within it.
+    if isinstance(content, PlaylistConfigContent):
+        for playlist in content.playlists:
+            add_selectors_to_tags(
+                playlist, tags_tracks, collection, auto_playlists
+            )
+        return
+
+    # This is not a folder so these playlists must have their selectors parsed.
+    numerical_selector_regex = re.compile(r"(?<=\[)[^\[\]]*(?=\])")
+    numerical_value_lookup = {}
+    numerical_value_set = set()
+    string_selector_regex = re.compile(r"(?<={)[^{}]+:[^{}]+(?=})")
+    string_selector_type_map = {
+        "artist": "get_artists",
+        "comment": "get_comments",
+        "date": "get_date_added",
+        "key": "get_key",
+        "label": "get_label",
+    }
+    string_value_lookup = {}
     playlists = set()
-    values_set = set()
-    bpm_rating_lookup = {}
-    playlist_selector_regex = r"(?<={)[^{}]*(?=})"
-    bpm_rating_selector_regex = r"(?<=\[)[^\[\]]*(?=\])"
 
-    # Grab selectors from Combiner playlists' names.
-    for playlist in playlist_config.combiner.playlists:
-        playlists.update(re.findall(playlist_selector_regex, playlist))
-        bpm_rating_matches = re.findall(bpm_rating_selector_regex, playlist)
-        if not bpm_rating_matches:
-            continue
-        parsed_bpms, parsed_ratings = parse_bpms_and_ratings(
-            bpm_rating_matches, bpm_rating_lookup
+    # Grab selectors from Combiner playlist name.
+    numerical_value_set.update(
+        parse_numerical_selectors(
+            re.findall(numerical_selector_regex, content),
+            numerical_value_lookup,
         )
-        values_set.update(parsed_bpms)
-        values_set.update(parsed_ratings)
+    )
+    parse_string_selectors(
+        re.findall(string_selector_regex, content),
+        string_value_lookup,
+        string_selector_type_map,
+        playlists,
+    )
 
-    # Add keys for BPM and rating selectors for tracks having those values.
-    for track_id, track in collection.get_tracks().items():
-        values = map(str, [round(track.get_bpm()), track.get_rating()])
-        for val in values:
-            if val not in values_set:
-                continue
-            for value, tag in bpm_rating_lookup.items():
+    # Add keys for numerical selectors for tracks having those values.
+    for value, tag in numerical_value_lookup.items():
+        if tag in tags_tracks:
+            continue
+
+        for track_id, track in collection.get_tracks().items():
+            values = map(
+                str, [round(track.get_bpm()), track.get_rating(), track.get_year()]
+            )
+            for val in values:
                 if (
                     (isinstance(value, str) and value == val) or
                     (isinstance(value, tuple) and val in value)
                 ):
                     tags_tracks[tag][track_id] = track
 
+    # Add keys for string selectors for tracks having those values.
+    for selector, tag in string_value_lookup.items():
+        if tag in tags_tracks:
+            continue
+
+        selector_type, selector_value = selector
+        for track_id, track in collection.get_tracks().items():
+            value = getattr(track, string_selector_type_map[selector_type])()
+            if not value:
+                continue
+            if selector_type == "date":
+                inequality, date, date_format = selector_value
+                if not inequality:
+                    if value.strftime(date_format) == date.strftime(date_format):
+                        tags_tracks[tag][track_id] = track
+                    continue
+                if not inequality(value, date):
+                    continue
+                tags_tracks[tag][track_id] = track
+                continue
+            if "*" in selector_value:
+                exp = re.compile(r".*".join(selector_value.lower().split("*")))
+                if re.search(exp, value.lower()):
+                    tags_tracks[tag][track_id] = track
+                    continue
+            if value.lower() == selector_value.lower():
+                tags_tracks[tag][track_id] = track
+
     # Get playlists for the identified playlist selectors. Not only must we get
     # playlists from the collection, but we must also get playlists from the
     # auto playlists constructed in the very same run of the playlist_builder.
     # This is because the playlists being selected may include those generated
     # by the playlist_builder.
     for playlist_name in playlists:
+        playlist_key = f"{{playlist:{playlist_name}}}"
+        if playlist_key in tags_tracks:
+            continue
+
         for playlist_object in [collection, *auto_playlists]:
             for playlist in playlist_object.get_playlists(playlist_name):
-                tags_tracks[f"{{{playlist_name}}}"].update(
+                tags_tracks[playlist_key].update(
                     playlist.get_tracks()
                 )
 
 
-def parse_bpms_and_ratings(
-    bpm_rating_match: List[str],
-    bpm_rating_lookup: Dict[Union[str, Tuple], str],
-) -> List[Tuple]:
-    """Parses a string match of one or more BPM and / or rating selectors.
+def parse_numerical_selectors(
+    numerical_matches: List[str],
+    numerical_value_lookup: Dict[Union[str, Tuple], str],
+) -> Set[str]:
+    """Parses a string match of one or more numerical selectors.
 
     Args:
-        bpm_rating_match: List of BPM or rating strings.
-        bpm_rating_lookup: Empty dict to populate with tuples or strings
-            mapping bpm or rating ranges or values to their "tag"
-            representation.
+        numerical_matches: List of numerical strings.
+        numerical_value_lookup: Empty dict to populate with tuples or strings
+            mapping numerical ranges or values to their "tag" representation.
 
     Returns:
-        Tuple of BPM and rating lists.
+        Set of numerical selector values.
     """
-    bpms, ratings = [], []
-    for bpm_rating in bpm_rating_match:
-        parts = map(str.strip, bpm_rating.split(","))
-        for part in parts:
-            number = _range = None
-            # If "part" is a digit, then it's an explicit BPM or rating to
-            # filter for.
-            if part.isdigit():
-                number = int(part)
-                if 0 <= number <= 5:
-                    ratings.append(str(number))
-                elif number > 5:
-                    bpms.append(str(number))
-            # If "part" is two digits separated by a "-", then it's a range
-            # of BPMs or ratings to filter for.
-            elif (
-                len(part.split("-")) == 2 and
-                all(x.isdigit() for x in part.split("-"))
+    numerical_values = set()
+    for match in numerical_matches:
+        _range = None
+        # If "match" is a digit, then it's an explicit numerical value.
+        if match.isdigit():
+            numerical_values.add(match)
+        # If "match" is two digits separated by a "-", then it's a range.
+        elif (
+            len(match.split("-")) == 2 and
+            all(x.isdigit() for x in match.split("-"))
+        ):
+            _range = list(map(int, match.split("-")))
+            _range = range(min(_range), max(_range) + 1)
+            if not (
+                all(0 <= x <= 5 for x in _range) or    # range for ratings
+                all(6 <= x <= 999 for x in _range) or  # range for BPMs
+                all(x >= 1000 for x in _range)         # range for years
             ):
-                _range = list(map(int, part.split("-")))
-                _range = range(min(_range), max(_range) + 1)
-                if all(0 <= x <= 5 for x in _range):
-                    ratings.extend(map(str, _range))
-                elif all(x > 5 for x in _range):
-                    bpms.extend(map(str, _range))
-                else:
-                    logger.error(f"Bad BPM or rating number range: {part}")
-                    continue
-            else:
-                logger.error(
-                    f"Malformed BPM or rating filter part: {part}"
-                )
+                logger.error(f"Bad numerical range selector: {match}")
+                continue
+            numerical_values.update(map(str, _range))
+        else:
+            logger.error(
+                f"Malformed numerical selector: {match}"
+            )
+            continue
+
+        numerical_value_lookup[
+            tuple(map(str, _range or [])) or match
+        ] = f"[{match}]"
+
+    return numerical_values
+
+
+def parse_string_selectors(
+    string_matches: List[str],
+    string_value_lookup:  Dict[Union[str, Tuple], str],
+    string_selector_type_map: Dict[str],
+    playlists: Set(str),
+):
+    """Parses a string match of one or more string selectors.
+
+    Args:
+        string_matches: List of strings for string selectors.
+        string_value_lookup: Empty dict to populate with strings mapping string
+            selectors to their "tag" representation.
+        string_selector_type_map: Maps a selector type to a Track method name.
+        playlists: Set for storing playlist names.
+    """
+    date_selector_regex = re.compile(r"(>=|>|<=|<)")
+    date_formats = ["%Y-%m-%d", "%Y-%m", "%Y"]
+    inequality_map = {
+        ">": lambda x, y: x > y,
+        "<": lambda x, y: x < y,
+        ">=": lambda x, y: x >= y,
+        "<=": lambda x, y: x <= y,
+    }
+
+    for match in string_matches:
+        selector_type, selector_value = map(str.strip, match.split(":"))
+        if selector_type == "playlist":
+            playlists.add(selector_value)
+            continue
+        if not string_selector_type_map.get(selector_type):
+            logger.warning(f"{selector_type} is not a supported selector!")
+            continue
+        if selector_type != "date":
+            string_value_lookup[(selector_type, selector_value)] = (
+                f"{{{match}}}"
+            )
+            continue
+
+        dates, formats, inequalities = [], [], []
+        skip_date_selector = False
+        for part in filter(
+            None, re.split(date_selector_regex, selector_value)
+        ):
+            if re.search(date_selector_regex, part):
+                inequalities.append(inequality_map[part])
                 continue
 
-            bpm_rating_lookup[
-                tuple(map(str, _range or [])) or str(number)
-            ] = f"[{part}]"
+            date = None
+            for date_format in date_formats:
+                try:
+                    date = datetime.strptime(part, date_format)
+                except ValueError:
+                    continue
+                break
+            if not date:
+                skip_date_selector = True
+                break
+            dates.append(date)
+            formats.append(date_format)
 
-    return bpms, ratings
+        if (
+            skip_date_selector or
+            len(dates) != 1  or
+            (len(inequalities) not in [0, 1])
+        ):
+            logger.warning(f"Date selector {selector_value} is invalid!")
+            continue
+
+
+        string_value_lookup[
+            (
+                selector_type,
+                (
+                    None if not inequalities else inequalities[0],
+                    dates[0],
+                    formats[0]
+                )
+            )
+        ] = f"{{{match}}}"
 
 
 def build_combiner_playlists(
-    playlist_config: PlaylistConfig,
+    content: Union[PlaylistConfig, str],
     tags_tracks: Dict[str, Dict[str, Track]],
     playlist_class: Playlist,
-) -> Playlist:
-    """Creates Combiner playlists.
+) -> Optional[Playlist]:
+    """Recursively traverses a playlist config to generate playlists from tags.
 
     Args:
-        playlist_config: PlaylistConfig object.
+        content: A component of a playlist config to create a playlist for.
         tags_tracks: Dict of tags to tracks.
         playlist_class: Playlist implementation class.
 
+    Raises:
+        ValueError: The user's playlist config must not be malformed.
+
     Returns:
-        Combiner playlists.
+        A Playlist or None.
     """
-    return playlist_class.new_playlist(
-        name=playlist_config.combiner.name, playlists=[
-            playlist_class.new_playlist(
-                name=expression,
-                tracks=parse_expression(expression, tags_tracks),
-            )
-            for expression in playlist_config.combiner.playlists
-        ]
-    )
+    if not isinstance(content, (PlaylistConfigContent, str)):
+        raise ValueError(f"Invalid input type {type(content)}: {content}")
+
+    # This is not a folder so a playlist with tracks must be created.
+    if isinstance(content, str):
+        return playlist_class.new_playlist(
+            name=content, tracks=parse_expression(content, tags_tracks)
+        )
+
+    # This is a folder so create playlists for those playlists within it.
+    playlists = [
+        build_combiner_playlists(item, tags_tracks, playlist_class)
+        for item in content.playlists
+    ]
+    playlists = [playlist for playlist in playlists if playlist]
+    if not playlists:
+        logger.warning(
+            f'There were no playlists created from "{content.playlists}"'
+        )
+
+    return playlist_class.new_playlist(name=content.name, playlists=playlists)
 
 
 def parse_expression(
@@ -584,6 +586,8 @@ class BooleanNode:
         self._tags = []
         self._tracks = []
         self._tags_tracks = tags_tracks
+        self._numerical_selector_regex = re.compile(r"(?<=\[)[^\[\]]*(?=\])")
+        self._string_selector_regex = re.compile(r"(?<={)[^{}]+:[^{}]+(?=})")
 
     def _get_tracks(self, tag: str) -> Set[str]:
         """Gets set of track IDs for the provided tag.
@@ -598,7 +602,10 @@ class BooleanNode:
         Returns:
             Set of track IDs for the provided tag.
         """
-        if "*" in tag:
+        if "*" in tag and not (
+            re.search(self._numerical_selector_regex, tag) or
+            re.search(self._string_selector_regex, tag)
+        ):
             exp = re.compile(r".*".join(tag.split("*")))
             tracks = {}
             for key in self._tags_tracks:
@@ -705,7 +712,16 @@ def print_playlists_tag_statistics(combiner_playlists: Playlist) -> None:
     Args:
         combiner_playlists: Playlist object for Combiner playlists.
     """
-    for playlist in combiner_playlists:
+    playlists = []
+    playlist_stack = [combiner_playlists]
+    while playlist_stack:
+        item = playlist_stack.pop()
+        if item.is_folder():
+            playlist_stack.extend(item.get_playlists())
+            continue
+        playlists.append(item)
+
+    for playlist in playlists:
         tracks = playlist.get_tracks()
         if tracks:
             print(f"\n{playlist.get_name()} tag statistics:")
@@ -735,10 +751,10 @@ def scale_data(
 
     Args:
         data: Tag names to tag counts.
-        maximum: Upper bound for rescaled data.
+        maximum: Upper bound for re-scaled data.
 
     Returns:
-        Rescaled dictionary of tag names and tag counts.
+        Re-scaled dictionary of tag names and tag counts.
     """
     data_max = max(data.items(), key=itemgetter(1))[1]
 
